@@ -42,24 +42,6 @@
 #include "applet-dialogs.h"
 #include "nm-ui-utils.h"
 
-typedef struct {
-	NMApplet *applet;
-	NMDevice *device;
-	NMConnection *connection;
-} BtMenuItemInfo;
-
-static void
-bt_menu_item_info_destroy (gpointer data)
-{
-	BtMenuItemInfo *info = data;
-
-	g_object_unref (G_OBJECT (info->device));
-	if (info->connection)
-		g_object_unref (G_OBJECT (info->connection));
-
-	g_slice_free (BtMenuItemInfo, data);
-}
-
 static gboolean
 bt_new_auto_connection (NMDevice *device,
                         gpointer dclass_data,
@@ -71,77 +53,16 @@ bt_new_auto_connection (NMDevice *device,
 	return FALSE;
 }
 
-static void
-bt_menu_item_activate (GtkMenuItem *item, gpointer user_data)
-{
-	BtMenuItemInfo *info = user_data;
-
-	applet_menu_item_activate_helper (info->device,
-	                                  info->connection,
-	                                  "/",
-	                                  info->applet,
-	                                  user_data);
-}
-
-
-typedef enum {
-	ADD_ACTIVE = 1,
-	ADD_INACTIVE = 2,
-} AddActiveInactiveEnum;
-
-static void
-add_connection_items (NMDevice *device,
-                      GSList *connections,
-                      NMConnection *active,
-                      AddActiveInactiveEnum flag,
-                      GtkWidget *menu,
-                      NMApplet *applet)
-{
-	GSList *iter;
-	BtMenuItemInfo *info;
-
-	for (iter = connections; iter; iter = g_slist_next (iter)) {
-		NMConnection *connection = NM_CONNECTION (iter->data);
-		GtkWidget *item;
-
-		if (active == connection) {
-			if ((flag & ADD_ACTIVE) == 0)
-				continue;
-		} else {
-			if ((flag & ADD_INACTIVE) == 0)
-				continue;
-		}
-
-		item = applet_new_menu_item_helper (connection, active, (flag & ADD_ACTIVE));
-
-		info = g_slice_new0 (BtMenuItemInfo);
-		info->applet = applet;
-		info->device = g_object_ref (G_OBJECT (device));
-		info->connection = g_object_ref (connection);
-
-		g_signal_connect_data (item, "activate",
-		                       G_CALLBACK (bt_menu_item_activate),
-		                       info,
-		                       (GClosureNotify) bt_menu_item_info_destroy, 0);
-
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
-}
-
 static void
 bt_add_menu_item (NMDevice *device,
-                  guint32 n_devices,
+                  gboolean multiple__devices,
+                  GSList *connections,
                   NMConnection *active,
                   GtkWidget *menu,
                   NMApplet *applet)
 {
 	const char *text;
 	GtkWidget *item;
-	GSList *connections, *all;
-
-	all = applet_get_all_connections (applet);
-	connections = nm_device_filter_connections (device, all);
-	g_slist_free (all);
 
 	text = nm_device_bt_get_name (NM_DEVICE_BT (device));
 	if (!text)
@@ -154,7 +75,7 @@ bt_add_menu_item (NMDevice *device,
 	gtk_widget_show (item);
 
 	if (g_slist_length (connections))
-		add_connection_items (device, connections, active, ADD_ACTIVE, menu, applet);
+		applet_add_connection_items (device, connections, TRUE, active, NMA_ADD_ACTIVE, menu, applet);
 
 	/* Notify user of unmanaged or unavailable device */
 	item = nma_menu_device_get_menu_item (device, applet, NULL);
@@ -167,41 +88,21 @@ bt_add_menu_item (NMDevice *device,
 		/* Add menu items for existing bluetooth connections for this device */
 		if (g_slist_length (connections)) {
 			applet_menu_item_add_complex_separator_helper (menu, applet, _("Available"), -1);
-			add_connection_items (device, connections, active, ADD_INACTIVE, menu, applet);
+			applet_add_connection_items (device, connections, TRUE, active, NMA_ADD_INACTIVE, menu, applet);
 		}
 	}
-
-	g_slist_free (connections);
 }
 
 static void
-bt_device_state_changed (NMDevice *device,
-                         NMDeviceState new_state,
-                         NMDeviceState old_state,
-                         NMDeviceStateReason reason,
-                         NMApplet *applet)
+bt_notify_connected (NMDevice *device,
+                     const char *msg,
+                     NMApplet *applet)
 {
-	if (new_state == NM_DEVICE_STATE_ACTIVATED) {
-		NMConnection *connection;
-		NMSettingConnection *s_con = NULL;
-		char *str = NULL;
-
-		connection = applet_find_active_connection_for_device (device, applet, NULL);
-		if (connection) {
-			const char *id;
-			s_con = nm_connection_get_setting_connection (connection);
-			id = s_con ? nm_setting_connection_get_id (s_con) : NULL;
-			if (id)
-				str = g_strdup_printf (_("You are now connected to '%s'."), id);
-		}
-
-		applet_do_notify_with_pref (applet,
-		                            _("Connection Established"),
-		                            str ? str : _("You are now connected to the mobile broadband network."),
-		                            "nm-device-wwan",
-		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
-		g_free (str);
-	}
+	applet_do_notify_with_pref (applet,
+	                            _("Connection Established"),
+	                            msg ? msg : _("You are now connected to the mobile broadband network."),
+	                            "nm-device-wwan",
+	                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
 }
 
 static GdkPixbuf *
@@ -370,7 +271,7 @@ applet_device_bt_get_class (NMApplet *applet)
 
 	dclass->new_auto_connection = bt_new_auto_connection;
 	dclass->add_menu_item = bt_add_menu_item;
-	dclass->device_state_changed = bt_device_state_changed;
+	dclass->notify_connected = bt_notify_connected;
 	dclass->get_icon = bt_get_icon;
 	dclass->get_secrets = bt_get_secrets;
 	dclass->secrets_request_size = sizeof (NMBtSecretsInfo);
