@@ -44,6 +44,7 @@
 #include "nm-mobile-providers.h"
 #include "mb-menu-item.h"
 #include "nm-ui-utils.h"
+#include "nm-gvaluearray-compat.h"
 
 typedef struct {
 	NMApplet *applet;
@@ -226,7 +227,8 @@ cdma_act_to_mb_act (CdmaDeviceInfo *info)
 
 static void
 cdma_add_menu_item (NMDevice *device,
-                    guint32 n_devices,
+                    gboolean multiple_devices,
+                    GSList *connections,
                     NMConnection *active,
                     GtkWidget *menu,
                     NMApplet *applet)
@@ -234,15 +236,11 @@ cdma_add_menu_item (NMDevice *device,
 	CdmaDeviceInfo *info;
 	char *text;
 	GtkWidget *item;
-	GSList *connections, *all, *iter;
+	GSList *iter;
 
 	info = g_object_get_data (G_OBJECT (device), "devinfo");
 
-	all = applet_get_all_connections (applet);
-	connections = nm_device_filter_connections (device, all);
-	g_slist_free (all);
-
-	if (n_devices > 1) {
+	if (multiple_devices) {
 		const char *desc;
 
 		desc = nma_utils_get_device_description (device);
@@ -317,8 +315,6 @@ cdma_add_menu_item (NMDevice *device,
 			add_connection_item (device, NULL, item, menu, applet);
 		}
 	}
-
-	g_slist_free (connections);
 }
 
 static void
@@ -330,34 +326,23 @@ cdma_device_state_changed (NMDevice *device,
 {
 	CdmaDeviceInfo *info;
 
-	if (new_state == NM_DEVICE_STATE_ACTIVATED) {
-		NMConnection *connection;
-		NMSettingConnection *s_con = NULL;
-		char *str = NULL;
-
-		connection = applet_find_active_connection_for_device (device, applet, NULL);
-		if (connection) {
-			const char *id;
-
-			s_con = nm_connection_get_setting_connection (connection);
-			id = s_con ? nm_setting_connection_get_id (s_con) : NULL;
-			if (id)
-				str = g_strdup_printf (_("You are now connected to '%s'."), id);
-		}
-
-		applet_do_notify_with_pref (applet,
-		                            _("Connection Established"),
-		                            str ? str : _("You are now connected to the CDMA network."),
-		                            "nm-device-wwan",
-		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
-		g_free (str);
-	}
-
 	/* Start/stop polling of quality and registration when device state changes */
 	info = g_object_get_data (G_OBJECT (device), "devinfo");
 	check_start_polling (info);
 }
 
+static void
+cdma_notify_connected (NMDevice *device,
+                       const char *msg,
+                       NMApplet *applet)
+{
+	applet_do_notify_with_pref (applet,
+	                            _("Connection Established"),
+	                            msg ? msg : _("You are now connected to the CDMA network."),
+	                            "nm-device-wwan",
+	                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
+}
+
 static GdkPixbuf *
 cdma_get_icon (NMDevice *device,
                NMDeviceState state,
@@ -747,6 +732,7 @@ applet_device_cdma_get_class (NMApplet *applet)
 	dclass->new_auto_connection = cdma_new_auto_connection;
 	dclass->add_menu_item = cdma_add_menu_item;
 	dclass->device_state_changed = cdma_device_state_changed;
+	dclass->notify_connected = cdma_notify_connected;
 	dclass->get_icon = cdma_get_icon;
 	dclass->get_secrets = cdma_get_secrets;
 	dclass->secrets_request_size = sizeof (MobileHelperSecretsInfo);
