@@ -49,6 +49,7 @@
 #include "nma-marshal.h"
 #include "nm-mobile-providers.h"
 #include "nm-ui-utils.h"
+#include "nm-gvaluearray-compat.h"
 
 typedef enum {
     MM_MODEM_GSM_ACCESS_TECH_UNKNOWN     = 0,
@@ -282,7 +283,8 @@ gsm_act_to_mb_act (GsmDeviceInfo *info)
 
 static void
 gsm_add_menu_item (NMDevice *device,
-                   guint32 n_devices,
+                   gboolean multiple_devices,
+                   GSList *connections,
                    NMConnection *active,
                    GtkWidget *menu,
                    NMApplet *applet)
@@ -290,15 +292,11 @@ gsm_add_menu_item (NMDevice *device,
 	GsmDeviceInfo *info;
 	char *text;
 	GtkWidget *item;
-	GSList *connections, *all, *iter;
+	GSList *iter;
 
 	info = g_object_get_data (G_OBJECT (device), "devinfo");
 
-	all = applet_get_all_connections (applet);
-	connections = nm_device_filter_connections (device, all);
-	g_slist_free (all);
-
-	if (n_devices > 1) {
+	if (multiple_devices) {
 		const char *desc;
 
 		desc = nma_utils_get_device_description (device);
@@ -373,8 +371,6 @@ gsm_add_menu_item (NMDevice *device,
 			add_connection_item (device, NULL, item, menu, applet);
 		}
 	}
-
-	g_slist_free (connections);
 }
 
 static void
@@ -386,34 +382,23 @@ gsm_device_state_changed (NMDevice *device,
 {
 	GsmDeviceInfo *info;
 
-	if (new_state == NM_DEVICE_STATE_ACTIVATED) {
-		NMConnection *connection;
-		NMSettingConnection *s_con = NULL;
-		char *str = NULL;
-
-		connection = applet_find_active_connection_for_device (device, applet, NULL);
-		if (connection) {
-			const char *id;
-
-			s_con = nm_connection_get_setting_connection (connection);
-			id = s_con ? nm_setting_connection_get_id (s_con) : NULL;
-			if (id)
-				str = g_strdup_printf (_("You are now connected to '%s'."), id);
-		}
-
-		applet_do_notify_with_pref (applet,
-		                            _("Connection Established"),
-		                            str ? str : _("You are now connected to the GSM network."),
-		                            "nm-device-wwan",
-		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
-		g_free (str);
-	}
-
 	/* Start/stop polling of quality and registration when device state changes */
 	info = g_object_get_data (G_OBJECT (device), "devinfo");
 	check_start_polling (info);
 }
 
+static void
+gsm_notify_connected (NMDevice *device,
+                      const char *msg,
+                      NMApplet *applet)
+{
+	applet_do_notify_with_pref (applet,
+	                            _("Connection Established"),
+	                            msg ? msg : _("You are now connected to the GSM network."),
+	                            "nm-device-wwan",
+	                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
+}
+
 static GdkPixbuf *
 gsm_get_icon (NMDevice *device,
               NMDeviceState state,
@@ -499,7 +484,7 @@ unlock_pin_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 		msg = error ? error->message : NULL;
 
 	applet_mobile_pin_dialog_stop_spinner (info->dialog, msg);
-	g_warning ("%s: error unlocking with PIN: %s", __func__, error->message);
+	g_warning ("%s: error unlocking with PIN: %s", __func__, error ? error->message : "unknown");
 	g_clear_error (&error);
 }
 
@@ -522,7 +507,7 @@ unlock_puk_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 		msg = error ? error->message : NULL;
 
 	applet_mobile_pin_dialog_stop_spinner (info->dialog, msg);
-	g_warning ("%s: error unlocking with PIN: %s", __func__, error->message);
+	g_warning ("%s: error unlocking with PUK: %s", __func__, error ? error->message : "unknown");
 	g_clear_error (&error);
 }
 
@@ -555,8 +540,7 @@ unlock_dialog_response (GtkDialog *dialog,
 
 	code1 = applet_mobile_pin_dialog_get_entry1 (info->dialog);
 	if (!code1 || !strlen (code1)) {
-		g_warn_if_fail (code1 != NULL);
-		g_warn_if_fail (strlen (code1));
+		g_warn_if_fail (code1 != NULL && strlen (code1));
 		unlock_dialog_destroy (info);
 		return;
 	}
@@ -1223,6 +1207,7 @@ applet_device_gsm_get_class (NMApplet *applet)
 	dclass->new_auto_connection = gsm_new_auto_connection;
 	dclass->add_menu_item = gsm_add_menu_item;
 	dclass->device_state_changed = gsm_device_state_changed;
+	dclass->notify_connected = gsm_notify_connected;
 	dclass->get_icon = gsm_get_icon;
 	dclass->get_secrets = gsm_get_secrets;
 	dclass->secrets_request_size = sizeof (MobileHelperSecretsInfo);
