@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/* NetworkManager Wireless Applet -- Display wireless access points and allow user control
+/* NetworkManager Applet -- allow user control over networking
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
-#include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <net/ethernet.h>
 
@@ -47,6 +46,10 @@
 #include "applet-agent.h"
 #include "shell-watcher.h"
 
+#if WITH_MODEM_MANAGER_1
+#include <libmm-glib.h>
+#endif
+
 #define NM_TYPE_APPLET			(nma_get_type())
 #define NM_APPLET(object)		(G_TYPE_CHECK_INSTANCE_CAST((object), NM_TYPE_APPLET, NMApplet))
 #define NM_APPLET_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), NM_TYPE_APPLET, NMAppletClass))
@@ -64,7 +67,8 @@ typedef struct
 #define PREF_DISABLE_DISCONNECTED_NOTIFICATIONS   "disable-disconnected-notifications"
 #define PREF_DISABLE_VPN_NOTIFICATIONS            "disable-vpn-notifications"
 #define PREF_DISABLE_WIFI_CREATE                  "disable-wifi-create"
-#define PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE "suppress-wireless-networks-available"
+#define PREF_SUPPRESS_WIFI_NETWORKS_AVAILABLE     "suppress-wireless-networks-available"
+#define PREF_SHOW_APPLET                          "show-applet"
 
 #define ICON_LAYER_LINK 0
 #define ICON_LAYER_VPN 1
@@ -81,12 +85,9 @@ typedef struct
 	GObject parent_instance;
 
 	GMainLoop *loop;
-	DBusGConnection *bus;
 	DBusGConnection *session_bus;
 
-#if GLIB_CHECK_VERSION(2,26,0)
 	NMShellWatcher *shell_watcher;
-#endif
 	guint agent_start_id;
 
 	NMClient *nm_client;
@@ -95,30 +96,45 @@ typedef struct
 
 	GSettings *gsettings;
 
+#if WITH_MODEM_MANAGER_1
+	MMManager *mm1;
+	gboolean   mm1_running;
+#endif
+
+	gboolean visible;
+
 	/* Permissions */
 	NMClientPermissionResult permissions[NM_CLIENT_PERMISSION_LAST + 1];
 
 	/* Device classes */
-	NMADeviceClass *wired_class;
+	NMADeviceClass *ethernet_class;
 	NMADeviceClass *wifi_class;
 	NMADeviceClass *gsm_class;
 	NMADeviceClass *cdma_class;
+#if WITH_MODEM_MANAGER_1
+	NMADeviceClass *broadband_class;
+#endif
 	NMADeviceClass *bt_class;
 	NMADeviceClass *wimax_class;
+	NMADeviceClass *vlan_class;
+	NMADeviceClass *bond_class;
+	NMADeviceClass *team_class;
+	NMADeviceClass *bridge_class;
+	NMADeviceClass *infiniband_class;
 
 	/* Data model elements */
 	guint			update_icon_id;
 
 	GtkIconTheme *	icon_theme;
 	GdkPixbuf *		no_connection_icon;
-	GdkPixbuf *		wired_icon;
+	GdkPixbuf *		ethernet_icon;
 	GdkPixbuf *		adhoc_icon;
 	GdkPixbuf *		wwan_icon;
-	GdkPixbuf *		wireless_00_icon;
-	GdkPixbuf *		wireless_25_icon;
-	GdkPixbuf *		wireless_50_icon;
-	GdkPixbuf *		wireless_75_icon;
-	GdkPixbuf *		wireless_100_icon;
+	GdkPixbuf *		wifi_00_icon;
+	GdkPixbuf *		wifi_25_icon;
+	GdkPixbuf *		wifi_50_icon;
+	GdkPixbuf *		wifi_75_icon;
+	GdkPixbuf *		wifi_100_icon;
 	GdkPixbuf *		secure_lock_icon;
 #define NUM_CONNECTING_STAGES 3
 #define NUM_CONNECTING_FRAMES 11
@@ -136,6 +152,7 @@ typedef struct
 	GdkPixbuf *		mb_tech_edge_icon;
 	GdkPixbuf *		mb_tech_umts_icon;
 	GdkPixbuf *		mb_tech_hspa_icon;
+	GdkPixbuf *		mb_tech_lte_icon;
 	GdkPixbuf *		mb_roaming_icon;
 	GdkPixbuf *		mb_tech_3g_icon;
 
@@ -217,7 +234,8 @@ struct NMADeviceClass {
 	                                        gpointer callback_data);
 
 	void           (*add_menu_item)        (NMDevice *device,
-	                                        guint32 num_devices,
+	                                        gboolean multiple_devices,
+	                                        GSList *connections,
 	                                        NMConnection *active,
 	                                        GtkWidget *menu,
 	                                        NMApplet *applet);
@@ -229,6 +247,9 @@ struct NMADeviceClass {
 	                                        NMDeviceState old_state,
 	                                        NMDeviceStateReason reason,
 	                                        NMApplet *applet);
+	void           (*notify_connected)     (NMDevice *device,
+	                                        const char *msg,
+	                                        NMApplet *applet);
 
 	/* Device class is expected to return a *referenced* pixbuf, which will
 	 * be unrefed by the icon code.  This allows the device class to create
@@ -319,4 +340,23 @@ gboolean applet_wifi_connect_to_8021x_network (NMApplet *applet,
 gboolean applet_wifi_create_wifi_network (NMApplet *applet);
 gboolean applet_wifi_can_create_wifi_network (NMApplet *applet);
 
+typedef enum {
+	NMA_ADD_ACTIVE = 1,
+	NMA_ADD_INACTIVE = 2,
+} NMAAddActiveInactiveEnum;
+
+void applet_add_connection_items (NMDevice *device,
+                                  GSList *connections,
+                                  gboolean sensitive,
+                                  NMConnection *active,
+                                  NMAAddActiveInactiveEnum flag,
+                                  GtkWidget *menu,
+                                  NMApplet *applet);
+
+void applet_add_default_connection_item (NMDevice *device,
+                                         const char *label,
+                                         gboolean sensitive,
+                                         GtkWidget *menu,
+                                         NMApplet *applet);
+
 #endif
