@@ -127,11 +127,7 @@ ce_page_8021x_security_new (NMConnection *connection,
 	parent = CE_PAGE (self);
 	priv = CE_PAGE_8021X_SECURITY_GET_PRIVATE (self);
 
-#if GTK_CHECK_VERSION (3,1,6)
 	parent->page = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-#else
-	parent->page = gtk_vbox_new (FALSE, 6);
-#endif
 	g_object_ref_sink (G_OBJECT (parent->page));
 	gtk_container_set_border_width (GTK_CONTAINER (parent->page), 6);
 
@@ -189,14 +185,6 @@ validate (CEPage *page, NMConnection *connection, GError **error)
 	return valid;
 }
 
-static GtkWidget *
-nag_user (CEPage *page)
-{
-	CEPage8021xSecurityPrivate *priv = CE_PAGE_8021X_SECURITY_GET_PRIVATE (page);
-
-	return priv->security ? wireless_security_nag_user (priv->security) : NULL;
-}
-
 static void
 ce_page_8021x_security_init (CEPage8021xSecurity *self)
 {
@@ -230,5 +218,4 @@ ce_page_8021x_security_class_init (CEPage8021xSecurityClass *security_class)
 	object_class->dispose = dispose;
 
 	parent_class->validate = validate;
-	parent_class->nag_user = nag_user;
 }
