@@ -52,8 +52,7 @@ get_tech_name (guint32 tech)
 	switch (tech) {
 	case MB_TECH_1XRTT:
 		return _("CDMA");
-	case MB_TECH_EVDO_REV0:
-	case MB_TECH_EVDO_REVA:
+	case MB_TECH_EVDO:
 		return _("EVDO");
 	case MB_TECH_GSM:
 		return _("GSM");
@@ -69,8 +68,12 @@ get_tech_name (guint32 tech)
 		return _("HSUPA");
 	case MB_TECH_HSPA:
 		return _("HSPA");
+	case MB_TECH_HSPA_PLUS:
+		return _("HSPA+");
 	case MB_TECH_WIMAX:
 		return _("WiMAX");
+	case MB_TECH_LTE:
+		return _("LTE");
 	default:
 		break;
 	}
@@ -196,11 +199,7 @@ nm_mb_menu_item_init (NMMbMenuItem *self)
 {
 	NMMbMenuItemPrivate *priv = NM_MB_MENU_ITEM_GET_PRIVATE (self);
 
-#if GTK_CHECK_VERSION(3,1,6)
-        priv->hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-#else
-	priv->hbox = gtk_hbox_new (FALSE, 6);
-#endif
+	priv->hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	priv->desc = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (priv->desc), 0.0, 0.5);
 
