@@ -59,5 +59,30 @@ typedef enum  {
 	NMA_ERROR_GENERIC
 } NMAError;
 
+
+gboolean utils_char_is_ascii_print (char character);
+gboolean utils_char_is_ascii_digit (char character);
+gboolean utils_char_is_ascii_ip4_address (char character);
+gboolean utils_char_is_ascii_ip6_address (char character);
+gboolean utils_char_is_ascii_apn (char character);
+
+typedef gboolean (*UtilsFilterGtkEditableFunc) (char character);
+gboolean utils_filter_editable_on_insert_text (GtkEditable *editable,
+                                               const gchar *text,
+                                               gint length,
+                                               gint *position,
+                                               void *user_data,
+                                               UtilsFilterGtkEditableFunc validate_character,
+                                               gpointer block_func);
+
+void utils_setup_password_storage (NMConnection *connection,
+                                   const char *setting_name,
+                                   GtkWidget *passwd_entry,
+                                   const char *password_flags_name);
+void utils_update_password_storage (NMSetting *setting,
+                                    NMSettingSecretFlags secret_flags,
+                                    GtkWidget *passwd_entry,
+                                    const char *password_flags_name);
+
 #endif /* UTILS_H */
 
