@@ -49,11 +49,13 @@ struct _WirelessSecurity {
 	const char *default_field;
 	gboolean adhoc_compatible;
 
+	char *username, *password;
+	gboolean always_ask, show_password;
+
 	WSAddToSizeGroupFunc add_to_size_group;
 	WSFillConnectionFunc fill_connection;
 	WSUpdateSecretsFunc update_secrets;
 	WSValidateFunc validate;
-	WSNagUserFunc nag_user;
 	WSDestroyFunc destroy;
 };
 
@@ -77,15 +79,21 @@ void wireless_security_fill_connection (WirelessSecurity *sec,
 void wireless_security_update_secrets (WirelessSecurity *sec,
                                        NMConnection *connection);
 
-GtkWidget * wireless_security_nag_user (WirelessSecurity *sec);
-
 gboolean wireless_security_adhoc_compatible (WirelessSecurity *sec);
 
+void wireless_security_set_userpass (WirelessSecurity *sec,
+                                     const char *user,
+                                     const char *password,
+                                     gboolean always_ask,
+                                     gboolean show_password);
+void wireless_security_set_userpass_802_1x (WirelessSecurity *sec,
+                                            NMConnection *connection);
+
 WirelessSecurity *wireless_security_ref (WirelessSecurity *sec);
 
 void wireless_security_unref (WirelessSecurity *sec);
 
-GType wireless_security_get_g_type (void);
+GType wireless_security_get_type (void);
 
 /* Below for internal use only */
 
@@ -140,8 +148,5 @@ void ws_802_1x_update_secrets (WirelessSecurity *sec,
                                const char *combo_name,
                                NMConnection *connection);
 
-GtkWidget * ws_802_1x_nag_user (WirelessSecurity *sec,
-                                const char *combo_name);
-
 #endif /* WIRELESS_SECURITY_H */
 
